#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <ctype.h>
#include "turtle.h"

#define RT	0	// Right Turn
#define LT	1	// Left Turn
#define FD	2	// ForwarD
#define BK	3	// BacK
#define HT	4	// Hatch Turtle
#define KT	5	// Kill Turtle
#define PU	6	// Pen Up
#define PD	7	// Pen Down
#define PC	8	// Pen Colour
#define HOME	9	// Go Home
#define REPEAT	10	// Repeat commands in []
static int numArgs[] =	{1,1,1,1,0,0,0,0,3,0,0}; // num of args for types


// the val is the value returned, if it's a colour triple
// then, val,g,b = r,g,b triple
int parse(char *str,int *command, int *val, int *g, int *b, turtle *eggs[], int *me);
void doCommand(int command,int val,int g, int b, turtle *eggs[],int *me);
int cmdline(int argc,char *argv[],int *x,int *y,int *x2,int *y2,int *maxhatch,int *err);
char *getBlock(char *str);
char *rmtrail(char *str);
int repeat(char *str,turtle *eggs[],int *me,int *program,int *pc);
int commStrToType(char *str);
void runProgram(int *program,int pc,int len,turtle *eggs[],int *me);
int getIter(char *str);
void error(char *str);

int MAXHATCH = 0; // max num of turtles to hatch

int penDown = 1; // start with pen on paper
int colourR = 0; // and colour set to black
int colourG = 0;
int colourB = 0;
int xHome=128,yHome=128;
int Angle=HOME_ANGLE;
char *macroName=NULL;
int reportErrors=0;

int main(int argc,char *argv[])
{
int command;
int val,g,b;
char *v;
char *str;
turtle **eggs;
int me=0;
int x2,y2;

if (cmdline(argc,argv,&xHome,&yHome,&x2,&y2,&MAXHATCH,&reportErrors))
	{
	Angle = RadToDeg(atan(((double)(yHome-y2))/((double)(xHome-x2))));
	
	if (x2>=xHome && y2>=yHome) Angle = 270 + (90+Angle);
	if (x2>=xHome && y2<=yHome) Angle = Angle;
	if (x2<=xHome && y2>=yHome) Angle = 90 + (90+Angle);
	if (x2<=xHome && y2<=yHome) Angle = 180 + (Angle);
	} // orient angle to x2,y2

if ( (eggs = (turtle **)malloc(sizeof(turtle)*MAXHATCH)) == NULL) exit(-1);
if ( (eggs[me] = (turtle *)malloc(sizeof(turtle))) == NULL ) exit(-1);

home(eggs[me]);

while(!feof(stdin))
{
if ( (str=malloc(255)) == NULL ) exit(-1);
v = fgets(str,255,stdin); // use read and bufsplit here?
if (v!=NULL)
	{
	val=0;while (str[val]!='\0') {str[val]=toupper(str[val]);val++;} // toUpper(str)
	if (parse(str,&command,&val,&g,&b,eggs,&me)) // returns false for REPEAT
	doCommand(command,val,g,b,eggs,&me);
	}
free(str);
} // end while not eof

free(eggs);
error("Exiting");
return 1;
} // end main


int getIter(char *str)
{
// get number after REPEAT...
int n;
char *s;

if ( (s=malloc(255)) == NULL ) exit(-1);
if ( sscanf(str,"REPEAT %s",s) )
	{
	if (isdigit(s[0]) && str[0]!='\0')
		{
		n = strtod(s,NULL);
		free(s);
		return n;
		}
	else
		{
		free(s);
		return -1;
		}
	}
else
	{
	free(s);
	return 0; // if no number, don't do it.
	}
} // end get number of iterations to do


int parse(char *str,int *command, int *val, int *g, int *b, turtle *eggs[], int *me)
{
char *commStr;
int args;
int *program=NULL;
int pc=0;
char *rstr;
int len;

if ( (commStr=malloc(255)) == NULL ) exit(-1);

str[strlen(str)-1] = '\0';

if ( (args = sscanf(str,"%s %d %d %d",commStr,val,g,b)) )
	{
	args--; // ignore the %s	
	if (!(numArgs[*command=commStrToType(commStr)]==args) && commStrToType(commStr)!=REPEAT )
	*command=-1; 
	}
if (*command==-1)
	{
	error("Invalid command");
	free(commStr); // I'm Free!
	return 0;
	}
if (*command == REPEAT) // special case, obviously
	{
	len = getIter(str);
	if (len==-1)
		{
		free(commStr);
		error("Invalid number of iterations");
		return 0; // no number
		}
	rstr = getBlock(str);
	if (rstr==NULL || rstr[0] == '\0')
		{
		error("Invalid code block");
		free(commStr);
		return 0;
		}
	if ( ( program=malloc((strlen(rstr))*sizeof(int)) ) == NULL ) exit(-1); // a better size would be number of tokens in str, but, I have 128MB and choose speed over footprint :)
	if (repeat(rstr,eggs,me,program,&pc)) runProgram(program,pc,len,eggs,me);
	free(commStr);
	free(rstr);
	free(program);
	return 1;
	} // end if REPEAT command.
else
	{
	free(commStr);
	return 1;
	}
}


int commStrToType(char *str)
{
if (!strcmp("FD",str)) return FD; else
if (!strcmp("BK",str)) return BK; else
if (!strcmp("LT",str)) return LT; else
if (!strcmp("RT",str)) return RT; else
if (!strcmp("HT",str)) return HT; else
if (!strcmp("KT",str)) return KT; else
if (!strcmp("PU",str)) return PU; else
if (!strcmp("PD",str)) return PD; else
if (!strcmp("PC",str)) return PC; else
if (!strcmp("HOME",str)) return HOME; else
if (!strcmp("REPEAT",str)) return REPEAT; else
return -1;
}

void runProgram(int *program,int pc,int len,turtle *eggs[],int *me)
{
int i;
int a=0,b=0,c=0; // the 3 args
int com;
int n;

for (n=0;n<len;n++)
for (i=0;i<pc;i++)
	{
	com = program[i];
	if (numArgs[com]==1) 
		{
		a = program[++i];
		}
	if (numArgs[com]==2) 
		{
		a = program[i+=1];
		b = program[i+=2];
		}
	if (numArgs[com]==3) 
		{
		a = program[i+=1];
		b = program[i+=2];
		c = program[i+=3];
		}
	doCommand(com,a,b,c,eggs,me);
	}
} // end run it


int repeat(char *s,turtle *eggs[],int *me,int *program,int *pc)
{
// n [ command1 command2 .. commandn ] make a program..
char *command=NULL;
int i;
int args=-1;
char *curArg=NULL;
int type;

// REMEMBER this will prolly screw str up something chronic

if (s == NULL || s[0]=='\0') return 0;

while(s!=NULL && s[0]!='\0')
	{

#ifdef __hpux
	command = strchr(&s," ");
#else
	command = strsep(&s," ");
#endif
	if (command==NULL)
		{
		error("Invalid command name");
		return 0;
		}
	type = commStrToType(command);
//	if (type==-1 || type==REPEAT) // recursion just won't work
	if (type==-1)
		{
		error("Invalid command name");
		return 0;
		}
	
	program[*pc] = type; // 1st op is type
	(*pc)++;

	args = numArgs[type];
	for (i=0;i<args;i++)
		{
#ifdef __hpux
	curArg = strchr(&s," ");
#else
		curArg = strsep(&s," ");
#endif

		if (curArg==NULL)
			{
			error("Need more arguments");
			return 0;
			}
		
		if (!isdigit(curArg[strlen(curArg)]) ) curArg[strlen(curArg)] = '\0';
		if (isdigit(curArg[0])) // if numerical argument
			{
			program[*pc] = atoi(curArg);
			(*pc)++;
			}
		else
			{
			error("Argument is not a number");
			return 0;
			}
		} // end for args
} // end while
return 1;
} // end repeat actions..


char *getBlock(char *str)
{
// return whatever's between [ and ]
char *start,*end,*data;

if (str==NULL) return NULL;
data = strdup(str);

start = strchr(str,'[');
if (start==NULL) return NULL; // no [ marker
end = strchr(str,']');
if (end==NULL) return NULL; // no ] marker
if (start>=end) return NULL; // Apple ][ 

strcpy(data,start+1);
while ( isspace(data[0]) ) strcpy(data,data+1); // remove whitespace
end = strrchr(data,']');
data[end-data] = '\0';
data = rmtrail(data);

return data; // pointer to a new malloced string with code
} // end get code block from str



char *rmtrail(char *str)
{
int i;
if (str && 0 != (i = strlen(str)))
	{
	while (--i >= 0)
		if (!isspace(str[i])) break;
	str[++i] = '\0';
	}
return str;
}


void doCommand(int command, int val, int g, int b, turtle *eggs[], int *me)
{
switch(command)
	{
	case RT:
	turnRight(eggs[*me],val);
	break;
	case LT:
	turnLeft(eggs[*me],val);
	break;
	case FD:
	forward(eggs[*me],val);
	break;
	case BK:
	back(eggs[*me],val);
	break;
	case HT:
	if (*me < MAXHATCH)
	{
	if ( (eggs[(*me)+1] = (turtle *)malloc(sizeof(turtle))) == NULL ) exit(-1);
		{
		copyTurtle(eggs[(*me)+1],eggs[*me]);
		(*me)++;
		}
	}
	else
	fprintf(stderr,"\nToo many hatchlings error (max: %d)\n",MAXHATCH);
	break;
	case KT:
	if (*me > 0)
		{
		free(eggs[*me]);
		(*me)--;
		}
	else
	fprintf(stderr,"\nInfant mortality error\n");
	break;
	case PU:
	penDown = 0;
	break;
	case PD:
	penDown = 1;
	break;
	case PC:
	colourR = val;
	colourG = g;
	colourB = b;
	break;
	case HOME:
	eggs[*me]->angle = Angle;
	eggs[*me]->x = xHome;
	eggs[*me]->y = yHome;
	break;
	}
} // end doCommand


void error(char *str)
{
if (reportErrors)
	{
	fprintf(stderr,"%s\n",str);
	}
} // end report an error
