#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <stdio.h>
#include <stdlib.h>
#include <glib.h>
#include "drawtool.h"

GtkWidget *colourseldlg=NULL;
GtkWidget *easycoloursel=NULL;
extern GdkColor curColour;
extern GtkWidget *indicator;
extern int throughput;

static int cols[] =	{
			0,0,0,
			65535,0,0,
			0,65535,0,
			0,0,65535,
			65535,65535,0,
			0,65535,65535,
			65535,0,65535,
			65535,65535,65535,
			32768,0,0,
			0,32768,0,
			0,0,32768,
			32768,32768,0,
			0,32768,32768,
			32768,0,32768,
			32768,32768,32768,
			32768,32768,65535
			};


void createEasyColourWindow(GtkWidget *w,gpointer d)
{
GtkWidget *drawing_area,*vbox;

if (easycoloursel==NULL)
{
easycoloursel = gtk_window_new(GTK_WINDOW_TOPLEVEL);
gtk_window_set_policy((GtkWindow*)easycoloursel,0,0,0);
gtk_widget_set_uposition(easycoloursel,((GtkWidget *)d)->allocation.x+((GtkWidget *)d)->allocation.width,((GtkWidget *)d)->allocation.y);
gtk_window_set_title((GtkWindow *)easycoloursel,"Easy Colour");
gtk_signal_connect(GTK_OBJECT(easycoloursel),"destroy",GTK_SIGNAL_FUNC (closeEasyColour), NULL);
vbox = createVBox(easycoloursel);
drawing_area = gtk_drawing_area_new ();
gtk_drawing_area_size (GTK_DRAWING_AREA (drawing_area),(XNUMBER*(XSIZE+SPACING))+SPACING,(YNUMBER*(YSIZE+SPACING))+SPACING);
gtk_box_pack_start (GTK_BOX (vbox), drawing_area, TRUE, TRUE, 0);

gtk_signal_connect(GTK_OBJECT (drawing_area), "expose_event",
		  (GtkSignalFunc) draw_simple, NULL);
gtk_signal_connect(GTK_OBJECT (drawing_area), "button_press_event",
		  (GtkSignalFunc) click_in_simple, NULL);
gtk_widget_set_events(drawing_area, GDK_EXPOSURE_MASK|GDK_BUTTON_PRESS_MASK);
gtk_widget_show(drawing_area);
}
gtk_widget_show(easycoloursel);
} // end create window to draw in.


gint draw_simple(GtkWidget *widget,GdkEventExpose *event)
{
GdkColor c;
GdkColormap *colourmap;
int x,y;
int i=0;

colourmap = gdk_window_get_colormap(widget->window);

for(x=0;x<XNUMBER;x++)
{
for(y=0;y<YNUMBER;y++)
{
c.red   = cols[(i*3)+0];
c.green = cols[(i*3)+1];
c.blue  = cols[(i*3)+2];
gdk_color_alloc(colourmap,&c);

draw_rect(widget,0,1,(x*(XSIZE+SPACING))+SPACING,(y*(YSIZE+SPACING))+SPACING,(x*(XSIZE+SPACING)+XSIZE)+SPACING,(y*(YSIZE+SPACING)+YSIZE)+SPACING,&c);
i++;
}
}
return FALSE;
}


gint click_in_simple(GtkWidget *widget,GdkEventButton *event)
{
int i=0;
int x,y;
GdkColormap *colourmap;

if (event->button == 1)
	{
	for(x=0;x<XNUMBER;x++)
		{
		for(y=0;y<YNUMBER;y++)
			{
			if (
			    event->x >= (x*(XSIZE+SPACING))+SPACING &&
			    event->x <= (x*(XSIZE+SPACING)+XSIZE)+SPACING &&
	  		    event->y >= (y*(YSIZE+SPACING))+SPACING &&
	 		    event->y <= (y*(YSIZE+SPACING)+YSIZE)+SPACING
   			   )
   			{
			curColour.red   = cols[(i*3)+0];
			curColour.green = cols[(i*3)+1];
			curColour.blue  = cols[(i*3)+2];
			colourmap = gdk_window_get_colormap(widget->window);
			gdk_color_alloc(colourmap,&curColour);
			draw_indicator(indicator,NULL);
			} // end if hit
			i++;
			} // end y
		} // end x
	} // end if button
return TRUE;
} // end func


void closeEasyColour(void)
{
gtk_widget_hide(easycoloursel);
gtk_widget_destroy(easycoloursel);
easycoloursel = NULL; // so we know to make a new one later..
} // end close the easy colour box


void coloursel(GtkWidget *w, gpointer d)
{
GtkWidget *sel;

if (colourseldlg==NULL)
{ 
colourseldlg = gtk_color_selection_dialog_new("Select colour");
sel = GTK_COLOR_SELECTION_DIALOG(colourseldlg)->colorsel;
gtk_signal_connect(GTK_OBJECT(sel),"color_changed",(GtkSignalFunc)colour_changed_cb,(gpointer)sel);
gtk_signal_connect(GTK_OBJECT(colourseldlg),"destroy",GTK_SIGNAL_FUNC(close_colourdlg), NULL);
gtk_signal_connect(GTK_OBJECT(GTK_COLOR_SELECTION_DIALOG(colourseldlg)->ok_button),"clicked",GTK_SIGNAL_FUNC(close_colourdlg), NULL);

// Yeah, right...
gtk_widget_hide(GTK_COLOR_SELECTION_DIALOG(colourseldlg)->help_button);
gtk_widget_hide(GTK_COLOR_SELECTION_DIALOG(colourseldlg)->cancel_button);
// show it
gtk_widget_show(colourseldlg);
}
} // end deal with colour select button press


void colour_changed_cb(GtkWidget *widget,GtkColorSelection *sel)
{
gdouble colour[3];
GdkColormap *colourmap;

gtk_color_selection_get_color(sel,colour);
colourmap = gdk_window_get_colormap(widget->window);
curColour.red = (guint16)(colour[0]*65535.0);
curColour.green = (guint16)(colour[1]*65535.0);
curColour.blue = (guint16)(colour[2]*65535.0);
gdk_color_alloc(colourmap,&curColour);
draw_indicator(indicator,NULL);
} // end colour changed cb

void close_colourdlg(void)
{
gtk_widget_hide(colourseldlg);
gtk_widget_destroy(colourseldlg);
colourseldlg = NULL; // so we know to make a new one later..
} // end close the colour sel dlg


GtkWidget *addDrawBox(GtkWidget *box,GtkWidget *window,int xsize, int ysize,GtkSignalFunc draw_func,GtkSignalFunc click_func)
{
GtkWidget *drawing_area;

drawing_area = gtk_drawing_area_new();
gtk_drawing_area_size (GTK_DRAWING_AREA (drawing_area),xsize,ysize);
gtk_box_pack_start (GTK_BOX (box), drawing_area, TRUE, TRUE, 0);

gtk_signal_connect(GTK_OBJECT (drawing_area), "expose_event",
		  (GtkSignalFunc) draw_func, box->window);
gtk_signal_connect(GTK_OBJECT (drawing_area), "button_press_event",
		  (GtkSignalFunc) click_func, box->window);
gtk_widget_set_events(drawing_area, GDK_EXPOSURE_MASK|GDK_BUTTON_PRESS_MASK);
gtk_widget_show(drawing_area);
return drawing_area;
} // end create window to draw in.


gint draw_indicator(GtkWidget *widget,GdkEventExpose *event)
{
int t;

t = throughput;
throughput = 0; // UGUGUGUGHGUGH dirty kluge alert
draw_rect(widget,0,1,0,0,widget->allocation.width-2,widget->allocation.height-1,&curColour);
throughput = t;
return FALSE;
}


